/* RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and that
no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net

File: rwhois.c  Everything starts here. 
Developed by: Scott Williamson

*/


#define MAIN 1

#include "rwhois.h"
#include "error.h"

int
main(argc, argv)
int argc;
char *argv[];
{
    int finished = FALSE;
    char **quicklook;
    struct SERVER *this_server;
    int wret = 0;  /* return from wait routine */
    char cmdline[MAXBUFFER];
    int cret = 0;   /* return from contact attempt */
    char rcfile[MAXBUFFER]; /* system wide default rc file */
    char lrcfile[MAXBUFFER];    /* users rc file */
    char *home_env;     /* users home directory..from the environment*/
#ifndef __STDC__
    char *getenv();
#endif
    int comp_ret;



    /* save the command name (usually called "rwhois") */
    cmdname = argv[0];

    argc--, argv++;

    printf("\nReferral Whois (RWhois) spec version %s (InterNIC V-%s)\n",
        SPEC_VERSION, IMP_VERSION);

    if( argc < 1 )
    {
        help();
        exit(-1);
    }

    shots = DEFAULTSHOT;  /* go through the entire refeal list */
    verbose = WHISPER;  
    default_wait = DEF_WAITTIME;
    default_maxwait = DEF_MAXWAIT;
    default_port = DEF_PORT;
    default_load = 0;
    send_limit( DEF_LIMIT, LIMIT_SET); /* set init default limit request */

    extreferral = FALSE;
    extsee_also = FALSE;
    default_cache = OFF;
    control_pannel[CP_CACHE] = SET_ALWAYS|SET_SEND;
    default_holdconnect = ON;
    control_pannel[CP_HOLDCONNECT] = SET_ALWAYS|SET_SEND;
    forcing = FALSE;
    authority = FALSE;
    current_modes = MODE_NONE;
    to_file = TRUE;
    f_out = NULL;
    goodedit = REG_NONE;
    edit = OFF;
    display_cap = DISPLAY_NONE;
    punt_referral = FALSE;
    user = NULL;
    authstring = NULL;
    just_show_config = FALSE;
    preserve_temp = FALSE;;
    reg_only = FALSE;

    /* setup the sever base structure */
    SERVERINIT;



    /* look quickly thourgh the augv to see if things should be verbose */
    /* don't act on all options till defaults are setup                 */

    quicklook = argv;

    while( *quicklook)
    {
        if (!strcmp(*quicklook, "-v")) 
            verbose = TALK;
        else if (!strcmp(*quicklook, "-d"))
            verbose = SHOUT;
        else if(!strcmp(*quicklook, "-q"))
            verbose = QUIET; 
        ++quicklook;
    }


    /* seed the server list with default servers */

    cmdline_server( SERDEF1, REF_PRESET);
    cmdline_server( SERDEF2, REF_PRESET);
    cmdline_server( SERDEF3, REF_PRESET);


    /**************************************************************/
    /* grab the defaults frist then process the command line  ..  */
    /* command line overrides defaults ... you will get a chance  */
    /* to run a different config file to modify defaults again    */
    /**************************************************************/


    /* first try the system wide rc file */
    sprintf( rcfile, "%s/rwhois.rc", CONFPATH);
    /* Try to process it */
    procrc( rcfile);

    /* Now try to open the user rc file and override anything needed */
    /* use the HOME environmential variable */
    if(!(home_env = getenv( "HOME")))
    {
        if( verbose > TALK )
            error("No HOME environment variable");
    }
    else
    {
        sprintf( lrcfile, "%s/.rwhois.rc", home_env);
        procrc( lrcfile);
    }

    /* select options and build the command list for whois/rwhois server */
    procargs( argv, cmdline);

    /* update the query for each of the default servers */
    upd_query( cmdline);

    if( !forcing )
        control_pannel[CP_QUERY] = (SET_SEND|SET_ALWAYS);

    /* if verbose mode show the user the final configuration */
    if( verbose > WHISPER || just_show_config)
    {
        printf("\nFinal configuration list:");
        list_defaults();
    }

    if( just_show_config)
        exit( 0);

    current_referral = 0;


    /* set server pointer to current server */
    this_server = SERVERCURRGET;

    /*  Start of the main query loop */

    while( !finished  )
    {
        if((cret = contact( this_server)) == CONTGOOD)
        {

            /* server after connection */
            this_server = SERVERCURRGET;

            if( verbose > WHISPER)
            {
                printf("\nConnnection made to %s port %d connection took %s\n",
                    this_server->server, this_server->port, 
                    showtime( this_server->start_time, this_server->con_time));
            }


            if( wait_result( default_wait) == WAIT_IDYES)
                server_setup();


            if( !forcing && !reg_only)
            {
                send_query( this_server->query);
                wret = wait_result( default_wait);
                if( edit == ON && wret != WAIT_REFERRAL )
                {
            
                    if( wret == WAIT_EDIT)
                        get_edit();

                    closeout( this_server);

                    if((comp_ret = files_the_same( this_server->filename,
                        this_server->bkupname)) == FALSE)
                    {
                        if((cret = contact( this_server)) == CONTGOOD)
                        {
                            if( wait_result( default_wait) == WAIT_IDYES)
                                server_setup();
                            if(send_register( goodedit, user, 
                                authstring) == GOOD)
                                wret = wait_result( default_wait);
                        }
                        else
                            error( "Unable to recontact server");
                    }
                    else if ( comp_ret == TRUE )
                        error("No change to object ...  data NOT sent");
                    else if( comp_ret == ERROR )
                        error( "Problems with temp file compare ");
                }
            }
            else if( reg_only)
            {
                if(send_register( regtype, user, 
                            authstring) == GOOD)
                    wret = wait_result( default_wait);
            }

            closeout( this_server);

            if( wret == WAIT_REFERRAL || extsee_also 
                || wret == WAIT_NO || wret == WAIT_CONTINUE )
            {
                if( wret != WAIT_REFERRAL && extsee_also)
                    --extsee_also;
                    
                this_server = SERVERCURRNEXT;
                punt_referral = FALSE;
            }
            else
            {
                finished = TRUE;
            }

        }
        else
            if( !(this_server = SERVERCURRNEXT))
                finished = TRUE;

    }

    if( verbose > WHISPER)
        server_list( LISTBACK, stdout);

    this_server = SERVERHEAD;

    do{
        if((this_server->filestatus&FILEHASDATA) 
            &&  !(this_server->reftype&REF_SEEALSO) 
            && edit != ON)
        {
            display_results( this_server->filename);    
        }
        /* remove the temp file otherwise leave it as local cache */
        if(!preserve_temp)
            unlink( this_server->filename);

        if( edit == ON )
        {
            if(!preserve_temp)
                unlink( this_server->bkupname);
        }

    } while( this_server = SERVERCURRNEXT);

    putchar('\n');
    fflush(stdout);
    return 0;
}

