/* RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and that
no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


*/
#include "rwhois.h"

help() 
{
    printf("\n%s: %s [-h hostname[:port] ] [ -a hostname[:port] ] ", 
            cmdname, cmdname); 
    printf("[-c config name ]\n\t\t[-f directive] [-u userid] [-x extended directive] [-s -m -v -q] question\n");
    printf("\n\t-h replace server list [ may be many ]");
    printf("\n\t-a add to server list [ may be many ]");
    printf("\n\t-c user config named [ may be many .. loaded in order ]");
    printf("\n\t\t[will try to add .rc to name if cant open]");
    printf("\n\t-s single server request .. no referral");
    printf("\n\t-m multiple referal");
    printf("\n\t-v verbose info to user");
    printf("\n\t-d debug ( very verbose )");
    printf("\n\t-q quiet ( not verbose at all )");
    printf("\n\t-f force directive");
    printf("\n\t-x extended directive");
    printf("\n\t-u user id for registration\n");
    printf("\n\t-p authstring used for registration\n");

}


upper( string )
char *string;
{
char *ptr;


        ptr = string;
        while(*ptr)
        {
        /*  do upper for all system even ones with toupper( ) problems */
            if( *ptr > 0x60 && *ptr < 0x7b )
                *ptr = *ptr - 0x20;
             ++ptr;
        }
}

int
fdata( buffer, maxlen, fd, wait)
char *buffer;
int maxlen;
int fd;
int wait;
{

char *ptr;
int count = 0;
char cbuf = 0;
int ret;
long time_in, time_curr, marktime();
int timeout = FALSE;
int error = FALSE;

#ifdef DEBUG
printf("\nEntering the data fetch with a wait of %d\n", wait);
fflush(stdout);
#endif

    fflush(stdout);
    ptr = buffer;
    time_in = marktime();

    *ptr = NullCH;

    do {


        if( wait)
        {
            time_curr = marktime();

            if(( time_curr - time_in) > wait) 
            {
                timeout = TRUE;
#ifdef DEBUG
printf("\nWait time reached must return\n");
fflush(stdout);
#endif
            }
        }

        ret = read( fd, &cbuf, 1);

        if( ret > 0 )
        {
            *(ptr++) = cbuf;
            ++count;
        }
        else if( ret < 0 )
        {
            if( errno == EWOULDBLOCK )
                continue;
            error = TRUE;
            if( errno != ECONNRESET)
                printf("Error: %s", strerror(errno));
        }
        else if( ret == 0 )
        {
                *buffer = NullCH;
                count = maxlen;
        }



    } while( cbuf != '\r'  && cbuf != '\n' && count != maxlen 
        && !timeout && !error);


    *ptr = NullCH;


    if( timeout)
        return( TIMEOUT);
    else    
        return( strlen( buffer));

}

long marktime()
{

struct timeval mark;

    gettimeofday( &mark, (struct timezone *)0);

    return( mark.tv_sec);

}

char *showtime( start, stop)
long start, stop;
{

static char diff[20];

    sprintf( diff, "%5.2f second%c", (double ) (stop - start), (stop - start) ? 's' : ' ');
    return( diff);


}   

list_defaults()
{
int x;

    printf("\nClient defaults:");
    printf("\nDefault wait : %d", default_wait);
    printf("\nDefault limit: %d", send_limit( 0, LIMIT_DEFAULT));
    printf("\nDefault load : %d %s", default_load,
        default_load ? "" : "Don't care");
    printf("\nDefault port : %d", default_port);
    printf("\nCaching      : %s", default_cache == ON ? "ON" : "OFF");
    printf("\nForwarding   : %s", default_forward == FORWARD_ON ? "ON" : "OFF");
    printf("\nVerbose level: ");
    switch( verbose )
    {
        case WHISPER:
            printf("whisper");
            break;
        case QUIET:
            printf("quiet");
            break;
        case TALK:
            printf("talk");
            break;
        case SHOUT:
            printf("shout");
            break;
        default:
            printf("unknow");
            break;
    }
    printf("\nConfiguration directory: %s", confdir ? confdir : "Not set");
    printf("\nLocal cache directory: %s", cachedir ? cachedir : "Not set");
    printf("\nEditor: %s", editpath ? editpath : "Not set");
    printf("\nURL follower: %s", urlpath ? urlpath : "Not set");
    printf("\nUser id: %s", user ? user : "Not set");

    printf("\nDirective flags:");
        for( x=0; x < PANNEL_COUNT; ++x)
            printf("\n\t[%d]%s: %x", x, dirstrings[x], (control_pannel[x]&0xff));

    server_list(LISTALL, stdout);
}

send_command( command)
char *command;
{
char outbuff[80];
struct SERVER *server;


    sprintf( outbuff, "%s%s\r\n", *command == '-' ? "" : "-",  command);

    if( verbose > WHISPER )
        printf("\nSending [%s] instruction to the server\n", command);

    server = SERVERCURRGET;

    write( server->sh, outbuff, strlen(outbuff));


}

send_extended( cmd)
char *cmd;
{
char *tmp;

    forcing = TRUE;
    if(!(strncmp( cmd, "-X-", 3)) || !(strncmp( cmd, "X-", 2)))
        send_command( cmd);
    else
    {
        tmp = safecalloc( 1, (strlen( cmd)+3));
        sprintf( tmp, "-X-%s", cmd);
        send_command( tmp);
        free( tmp);
    }
}


send_query( question)
char *question;
{
char outbuff[80];
char final_out[80];
struct SERVER *server;


    strcpy( outbuff, question);

/* add the query optoins here if need */

    if( verbose > WHISPER )
        printf("\nSending query [%s] to server\n", outbuff);

    if( edit )
        sprintf( final_out, "dump %s\r\n", outbuff);
    else
        sprintf( final_out, "%s\r\n", outbuff);

    server = SERVERCURRGET;

    write( server->sh, final_out, strlen(final_out));
    control_pannel[CP_QUERY] = (control_pannel[CP_QUERY]|SET_WAIT);

}


send_limit( value, action)
int value, action;
{
static int limit_set;
static int limit_use;
char outbuff[80];


    if( action == LIMIT_SET )
    {
        limit_set = value;
        return(0);
    }
    else if( action == LIMIT_SEND)
        limit_use = limit_set;
    else if( action == LIMIT_CURRENT)
        return( limit_use);
    else if( action == LIMIT_DEFAULT)
        return( limit_set);
    else if( action == LIMIT_RESEND)
        limit_use = limit_use/2;

#ifdef DEBUG
printf("\nTrying limit of %d", limit_use);
fflush(stdout);
#endif

    sprintf( outbuff, "-LIMIT %d", limit_use);
    send_command( outbuff);
    return(0);
}

send_cache( action)
int action;
{
    if( action == ON)
        send_command("-CACHE ON");
    else
        send_command("-CACHE OFF");
}

send_holdconnect( action)
int action;
{
    if( action == ON)
        send_command("-HOLDCONNECT ON");
    else
        send_command("-HOLDCONNECT OFF");
}

send_soa( area)
char *area;
{
char *tmp;
    
        if( area)
        {
            tmp = safecalloc( 1,(strlen( area) + 6));
            sprintf( tmp, "-SOA %s", area ? area : "");
            send_command( tmp);
            free( tmp);
        }
        else
            send_command( "SOA");

}

send_directive( name)
char *name;
{
char *tmp;
    
        if( name)
        {
            tmp = safecalloc( 1,(strlen( name) + 12));
            sprintf( tmp, "-DIRECTIVE %s", name ? name : "");
            send_command( tmp);
            free( tmp);
        }
        else
            send_command("DIRECTIVE");
}

send_define( name)
char *name;
{
char *tmp;
    
        if( name)
        {
            tmp = safecalloc( 1,(strlen( name) + 9));
            sprintf( tmp, "-define %s", name ? name : "");
            send_command( tmp);
            free( tmp);
        }
        else
            send_command("DEFINE");
}

send_display( type)
char *type;
{
char *tmp;
static int sent;
    
        if(type)
        {
            tmp = safecalloc( 1,(strlen( type) + 10));
            sprintf( tmp, "-display %s", type ? type : "");
            send_command( tmp);
            free( tmp);
        }
        else
        {
            if( display_cap&DISPLAY_MIME && !(sent&DISPLAY_MIME))
            {   
                send_command("DISPLAY CAPABLE MIME");
                sent = sent|DISPLAY_MIME;
            }
            else if( display_cap&DISPLAY_URL && !(sent&DISPLAY_URL))
            {
                send_command("DISPLAY CAPABLE URL");
                sent = sent|DISPLAY_URL;
            }

            if( display_cap == sent )
            {
                sent = 0;
                return( 0);
            }
            else
                return(-1);

        }

        return( 0);
}


send_xfer( obj)
char *obj;
{
char *tmp;
    

        if( obj)
        {
            tmp = safecalloc( 1,(strlen( obj) + 7));
            sprintf( tmp, "-XFER %s", obj ? obj : "");
            send_command( tmp);
            free( tmp);
        }
        else
            send_command("XFER");
    
}

send_object( objname)
char *objname;
{
char *tmp;
    

        if( objname)
        {
            tmp = safecalloc( 1,(strlen( objname) + 9));
            sprintf( tmp, "-OBJECT %s", objname ? objname : "");
            send_command( tmp);
            free( tmp);
        }
        else
            send_command("OBJECT");
    
}


send_register( action, who, authstring)
int action;
char *who;
char *authstring;
{
char act_string[4];
char *temp_in;
int ret = GOOD;

        switch( action)
        {

            case REG_MOD:
                strcpy( act_string, "MOD");
                break;
            case REG_DEL:
                strcpy( act_string, "DEL");
                break;
            case REG_ADD:
                strcpy( act_string, "ADD");
                break;
            default:
                ret = BAD; 
        }

        if( ret == GOOD)
        {   
            if( who && authstring )
            {
                temp_in = safecalloc( 1, 20 + strlen( act_string) + 
                    strlen( who) + strlen( authstring));
                sprintf( temp_in, "-REGISTER ON %s %s %s", act_string, 
                    who, authstring);
                send_command( temp_in);
                free( temp_in);
                control_pannel[CP_REGISTER] = 
                    control_pannel[CP_REGISTER]|(SET_ON|SET_WAIT);
            }
            else
            {
                error("Must have id and authstring of person doing registration");
                ret = BAD;
            }
        }
        else
            error("Bad register mode");

        return( ret);
}


send_schema( obj)
char *obj;
{
char *tmp;
    
    if( obj)
    {
        tmp = safecalloc( 1,(strlen( obj) + 7));
        sprintf( tmp, "-SCHEMA %s", obj ? obj : "");
        send_command( tmp);
        free( tmp);
    }
    else
        send_command("SCHEMA");
}

send_quit()
{
    send_command( "-QUIT");
    return( -1);
}

reset_flags()
{
int x;

    for( x=0; x <  PANNEL_COUNT; ++x)
        if( control_pannel[x] & SET_ALWAYS)
            control_pannel[x] = control_pannel[x]|SET_SEND;


}

stop_flags()
{
int x;

    for( x=0; x <  PANNEL_COUNT; ++x)
            control_pannel[x] = control_pannel[x]&~SET_SEND;

}

setsend( line)
char *line;
{
char temp[80];
int ret = 0;
int pannel_num = 0;


    upper( line);

    forcing = TRUE;

    /* SOA */
    if(!strncmp( line, "SOA", 3))
    {
        if((sscanf( line, "SOA %s", temp)) == 1)
        {
            pannel_num = CP_SOA;        
            autharea = safecalloc( 1, strlen( temp)+1);
            strcpy( autharea, temp);
        }
        else
            printf("\nNeed authority area for SOA query\n");
    }

    /*LIMIT*/
    else if(!strncmp( line, "LIMIT", 5))
    {
        if((sscanf( line, "LIMIT %s", temp)) == 1)
        {
            pannel_num = CP_LIMIT;
            send_limit( atoi( temp), LIMIT_SET);
        }
        else
            printf("\nNeed limit requested\n");
    }

    /*LOAD*/
    else if(!strncmp( line, "LOAD", 4))
        pannel_num = CP_LOAD;

    /*STATUS*/
    else if(!strncmp( line, "STATUS", 6))
        pannel_num = CP_STATUS;

    /*XFER*/
    else if(!strncmp( line, "XFER", 4))
    {
        pannel_num = CP_XFER;
        if((sscanf( line, "XFER %s", temp)) == 1)
        {
            xfer_object = safecalloc( 1, strlen( temp)+1);
            strcpy( xfer_object, temp);
        }
        else
            xfer_object = NULL;
    }

    /*OBJECT*/
    else if(!strncmp( line, "OBJECT", 6))
    {
        pannel_num = CP_OBJECT;
        if((sscanf( line, "OBJECT %s", temp)) == 1)
        {
            object_name = safecalloc( 1, strlen( temp)+1);
            strcpy( object_name, temp);
        }
        else
            object_name = NULL;
    }

    /*SCHEMA*/
    else if(!strncmp( line, "SCHEMA", 6))
    {
        pannel_num = CP_SCHEMA;
        if((sscanf( line, "SCHEMA %s", temp)) == 1)
        {
            schema_object = safecalloc( 1, strlen( temp)+1);
            strcpy( schema_object, temp);
        }
    }

    /*DIRECTIVE*/
    else if(!strncmp( line, "DIRECTIVE", 9))
    {
        pannel_num = CP_DIRECTIVE;
        if((sscanf( line, "DIRECTIVE %s", temp)) == 1)
        {
            directive_name = safecalloc( 1, strlen( temp)+1);
            strcpy( directive_name, temp);
        }
        else
            directive_name = NULL;
    }
    /*DEFINE*/
    else if(!strncmp( line, "DEFINE", 6))
    {
        pannel_num = CP_DEFINE;
        if((sscanf( line, "DEFINE %s", temp)) == 1)
        {
            define_name = safecalloc( 1, strlen( temp+1));
            strcpy( define_name, temp);
        }
        else
            define_name = NULL;

    }
    else
    {
        printf("\nNot a valid forced directive\n");
        ret = -1;
    }

    if ( pannel_num)
        control_pannel[pannel_num] = ( control_pannel[pannel_num] 
                | SET_SEND | SET_FORCE);
        
    return( ret);

}

trim( line)
char *line;
{
char *ptr;

    ptr = rindex( line, '\n');
    if( ptr)
        *ptr = NullCH;

}


brkresponse( parts, line, objname, attname, value)
int parts;
char *line, **objname, **attname, **value;
{
char *ptr;

        if((*objname = index( line, ' ')))
        {
            while( **objname == ' ' )
                ++(*objname);

            if((ptr = rindex( *objname, '\n' )))
                *ptr = NullCH;

            if((*attname = index( *objname, ':')))
            {
                    **attname = NullCH;
                    ++*attname;
                    while( **attname == ' ')
                        ++attname;

                if( parts == 3)
                {
                    if((*value= index( *attname, ':')))
                    {
                        **value = NullCH;
                        ++*value;
                        while( **value == ' ' )
                            ++value;
                        return( GOOD);
                    }
                }
                else
                    return( GOOD);

            }
        }

        return( BAD);

}

get_edit()
{
char temp[80];
char *out_buf;

    send_command("OBJECT");
    control_pannel[CP_OBJECT] = control_pannel[CP_OBJECT]&SET_WAIT;

    wait_result( default_wait);

    printf("\nEnter object name : ");
    fflush(stdin);
    gets( temp);
    gets( temp);
    control_pannel[CP_OBJECT] = 
            (control_pannel[CP_OBJECT]&~SET_WAIT);
    to_file = TRUE;
    
    out_buf = safecalloc( 1,(strlen( temp)+ 8));

    sprintf( out_buf, "SCHEMA %s", temp);
    send_command(out_buf);
    control_pannel[CP_SCHEMA] = SET_WAIT;

    free( out_buf);

    wait_result( default_wait);

}

nothing_else_todo()
{
int x;

#ifdef DEBUG
printf("\n Inside Nothing else to do function");
fflush(stdout);
#endif

    for( x=0; x <  PANNEL_COUNT; ++x)
            if((control_pannel[x] & SET_SEND) || 
                ( control_pannel[x] & SET_WAIT))
            {
#ifdef DEBUG
printf("\nBAD NOTHING ELSE TO DO [%d]", x);
fflush(stdout);
#endif
                return( BAD);
            }

#ifdef DEBUG
printf("\nGOOD NOTHING ELSE TO DO");
fflush(stdout);
#endif

    return( GOOD);

}

/* trim ( LF, CR, SPACE) form the end of a string */

trimstr( line)
char *line;
{
char *ptr;

    ptr = &line[strlen(line)];
    while( (ptr != line) && 
        ( *ptr == NullCH || *ptr == '\n' || *ptr == ' ' || *ptr == '\r'))
        --ptr;

    *(++ptr) = NullCH;
}

    

gourl( line)
char *line;
{
char *ptr;
char *obuffer;

    if( !urlpath)
    {
    error("Additional information not seen... URL follower not configured");
        return( ERROR);
    }

    if( ptr = index( line, ':'))
    {
        ++ptr;
        obuffer = safecalloc( 1, ( strlen( ptr) + strlen( urlpath) + 2));
        sprintf( obuffer, "%s %s", urlpath, ptr);
        system( obuffer);
        free( obuffer);
    }
    else
    {
        error("\nBad URL format... not sure why ");
        return( ERROR);
    }

    return( GOOD);
}

error( string)
char *string;
{
    trim( string);
    printf("\n<ERROR> %s\n", string);
}

message( string)
char *string;
{
    trim( string);
    printf("\n<MESSAGE> %s\n", string);
}

question( string)
char *string;
{
    trim( string);
    printf("\n<QUESTION> %s ? ", string);
}


note( string)
char *string;
{
    trim( string);
    printf("\n<NOTE> %s\n", string);

}

warning( string)
char *string;
{
    trim( string);
    printf("\n<WARNING> %s\n", string);

}

display_results( name)
char *name;
{
FILE *fp;
char buffer[MAXBUFF];
char temp[MAXBUFF];

    if(!(fp = fopen( name, "r")))
    {
        sprintf( temp, "Unable to open display file [%s]", name);
        error( temp);
    }
    else
    {
        if( verbose > TALK )
        {
            sprintf( buffer, "Query results from [%s]", name);
            message( buffer);
        }

        while(fgets( buffer, 256, fp))
        {
            if(strncmp( buffer, "#RWhois:", 7))
            {
                    if( post_response( buffer) == WAIT_NULL)
                    {
                        printf( "%s", buffer);
                        fflush(stdout);
                    }
            }
        }
        fclose( fp);
    }
}


display_second( name)
char *name;
{
FILE *fp;
char buffer[MAXBUFF];
char temp[MAXBUFF];

    if(!(fp = fopen( name, "r")))
    {
        sprintf( temp, "Unable to open secondary display file [%s]", name);
        error( temp);
    }
    else
    {
        while(fgets( buffer, MAXBUFF, fp))
        {
            if(strncmp( buffer, "#RWhois:", 7))
                printf( "%s", buffer);
        }
        fclose( fp);
    }
}


wkfile_write( wkserver, sout)
struct SERVER *wkserver;
char *sout;
{
char file_name[MAXBUFF];

    if( !wkserver->fp)
    {
        strcpy( file_name, FILENAMEMASK);
        mktemp( file_name);
        wkserver->filename = safecalloc( 1, (strlen(file_name)+1));
        strcpy( wkserver->filename, file_name);
        wkserver->fp = fopen( wkserver->filename, "w");
        wkserver->filestatus = (wkserver->filestatus
            |FILEOPEN);
        if( goodedit != REG_DEL)
            goodedit = REG_MOD;
        fprintf( wkserver->fp, "#RWhois://%s:%d:%s\n",
            wkserver->server, wkserver->port, wkserver->query);
        if( edit == ON)
        {
            wkserver->bkupname = safecalloc( 1, (strlen(wkserver->filename)+5));
            sprintf( wkserver->bkupname, "%s.org", wkserver->filename);
            wkserver->bkfp = fopen( wkserver->bkupname, "w");
            fprintf( wkserver->bkfp, "#RWhois://%s:%d:%s\n",
                wkserver->server, wkserver->port, wkserver->query);
        }

     }

    if( !(wkserver->filestatus&FILEHASDATA)
        && *sout != '\n' && *sout != NullCH && *sout != '\r')
    {       
            wkserver->filestatus = 
                (wkserver->filestatus|FILEHASDATA);
    }

    if(!(current_modes&MODE_INFO))
        fprintf( wkserver->fp, "%s", sout);
    else
        printf("%s", sout);

    if( edit  && !(current_modes&MODE_INFO))
        fprintf( wkserver->bkfp, "%s", sout);

    return( GOOD);
}

/* Donated by Stan Barber */

static char nomem[] = "rwhois: out of memory!\n";

/* paranoid version of calloc */

char *
safecalloc(number,size)
int number;
int size;
{
    char *ptr;
    ptr = (char *)calloc(number,size ? size : 1);
    if (ptr == NULL) {
    fputs(nomem,stdout);
    fflush(stdout);
    exit(-1);
    }
    return ptr;
}


