/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "client_msgs.h"

#include <stdio.h>
#include <varargs.h>

static struct err_struct {
  error_codes_type err_no;
  char *msg;
} errs[] =  {
  { FAILED_PEERNAME, "100 Get Peer Name Query Failed"},
  { REGIST_DEFFERED, "120 Registration Not Processed... Will Process Hours:"},
  { NON_AUTH_ANSWER, "130 Not Authority to Answer... TTL Good"},
  { HANDLE_GENERATED, "131 Handle Not Supplied. Generated Handle Follows:"},
  { REFERRED_SERVICE, "200 Service Not Available.. Referral To Follow"},
  { NO_RECORDS_FOUND, "230 No Records Found"},
  { NON_AUTH_MARTIAN, "231 Not Authority to Answer.. TTL Expired"},
  { UNAUTH_TO_CACHE,  "232 Cache Disabled"},
  { UNCOMPAT_VERSION, "300 Not Compatible With That Version Number"},
  { INVALID_ATTRIBUTE, "320 Invalid Attribute Line:"},
  { INVALID_FORMAT, "321 Invalid Format Line:"},
  { MISSING_REQ_ATTRIB, "322 Required Attribute Missing Name:"},
  { MISSING_REL_OBJECT, "323 Required Related Object Missing Name:"},
  { NON_UNIQ_KEY, "324 Primary Key Not Unique"},
  { EXCEEDED_MAXREC, "330 Exceeded Max Records Limit"},
  { INVALID_MAXREC, "331 Invalid Max Records Size"},
  { NO_TRANSFER, "332 Nothing To Transfer"},
  { NO_SOA, "333 Not SOA for Requested Authority Area"},
  { UNIMPLEM_DIRECTIVE, "334 Pre-query Directive Not Implemented"},
  { UNAVAIL_SYSLOAD, "335 Systems Load Not Available"},
  { NO_OBJECT_FOUND, "336 Object Not Found"},
  { NO_OBJECT_SCHEMA_FOUND, "337 Object's Schema Not Found"},
  { INVALID_DIRECTIVE_PARAM, "338 Invalid Directive Parameter"},
  { INVALID_DIRECTIVE, "400 Invalid Server Directive"},
  { UNAUTH_DIRECTIVE, "401 Not Authorized for Directive"},
  { UNIDENT_ERROR, "402 Unidentified Error .. Continue"},
  { UNAUTH_REGIST, "420 Registration Not Authorized"},
  { UNAUTH_REGIST_RECORD, "421 Not Authorized to Change Object"},
  { UNAUTH_TO_FRWD, "431 Not Authorized To Forward"},
  { UNAUTH_TO_XFER, "432 Not Authorized To Transfer"},
  { BAD_REF_ON_FRWD, "433 Bad Reference On Forward"},
  { NON_EXIST_REFER, "434 Referral Does Not Exist on this Server"},
  { UNDEF_DEFINE, "435 Macro not Defined"},
  { UNAUTH_DISPLAY, "436 Display Mode Not Allowed"},
  { UNSUPPORT_LANG, "437 Language Not Supported"},
  { UNKNOWN_X_DIRECTIVE, "460 Extended Directive Not Recognized"},
  { UNAUTH_X_DIRECTIVE, "461 Extended Directive Not Authorized"},
  { MEMORY_ALLOCATION_PROBLEM, "500 Memory Allocation Problem"},
  { SERVICE_NOT_AVAIL, "501 Service Not Available"},
  { UNRECOV_ERROR, "502 Unrecoverable Error... Goodbye"},
  { DEADMAN_TIME, "503 Idle Time Exceeded... Goodbye"},
  { UNAUTH_SECONDARY, "530 Not Authorized As Secondary"},
  { MISC_DIAG, "560 "}
};

#define N_ERRS (sizeof(errs)/sizeof (struct err_struct))

/* prints to stdout the error messages. Format: %error ### message
     text, where ### follows rfc 640 */
void
print_error(va_alist)
  va_dcl
{
  va_list   list;
  int       i;
  int       err_no;
  char      *format;

  va_start(list);

  err_no = va_arg(list, int);

  for (i = 0; i < N_ERRS; i++)
  {
    if (errs[i].err_no == err_no)
    {
      printf("%%error %s ", errs[i].msg);
      break;
    }
  }

  format = va_arg(list, char*);
  vprintf(format, list);
  va_end(list);

  printf("\n");
}

/* prints to stdout the ok message */
void print_ok ()
{
  printf ("%%ok\n");
}
