/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include "output.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */
#include "defines.h"
#include "strutil.h"
#include "misc.h"
#include "client_msgs.h"

/* ----------------------- LOCAL FUNCTIONS -------------------- */

/* determines the field type */
static output_format_type
get_format_type (type)
  char *type;
{
  if (STR_EQ(type, O_FIELD_DEF))
  {
    return (FORMAT_FIELD_DEF);
  }
  else if (STR_EQ(type, O_TEXT_DEF))
  {
    return (FORMAT_TEXT);
  }
  else if (STR_EQ(type, O_CONDIT_TEXT))
  {
    return (FORMAT_CONDIT_TEXT);
  }
  else if (STR_EQ(type, O_AUTH_ELEM))
  {
    return (FORMAT_AUTH_ELEM);
  }

  print_error (UNIDENT_ERROR, "get_format_type: type %s not found!", type);

  return (-1);
}


/* appends subitem to list */
static output_format_struct *
make_subitem(ptr, type_des, value)
  output_format_struct  *ptr;
  char                  *type_des;
  char                  *value;
{
  output_format_struct  *new;
  output_format_struct  *cur = ptr;

  new = NEW_OUTPUT_FORMAT;
  
  if (STR_EQ(type_des, O_TITLE))
  {
    new->type = FORMAT_TEXT;
  }
  else if (STR_EQ(type_des, O_DATA))
  {
    new->type = FORMAT_DATA;
  }
  else if (STR_EQ(type_des, O_SUB_FORMAT))
  {
    new->type = FORMAT_SUB_FORMAT;
  }
  else if (STR_EQ(type_des, O_AUTH_ELEM))
  {
    new->type = FORMAT_AUTH_ELEM;
  }

  new->value = NEW_STRING (value);

  if (cur == NULL)
  {
    return (new);
  }

  while (cur->nxt != NULL)
  {
    cur=cur->nxt;
  }

  cur->nxt = new;

  return (ptr);
}

/* saves the output to a structure */
static void
save_output_format (format_list, tag, datum)
  output_format_struct  *format_list;
  char                  *tag;
  char                  *datum;
{
  if (STR_EQ(tag, O_TYPE))
  {
    format_list->type = get_format_type(datum);
  }
  else if (STR_EQ(tag, O_FIELD))
  {
    format_list->value = NEW_STRING(datum);
  }
  else if (STR_EQ(tag, O_FIXED_LEN))
  {
    format_list->fixed_len = atoi(datum);
  }
  else if (STR_EQ(tag, O_SUB_DEREF))
  {
    format_list->sub_deref = true_false(datum);
  }
  else if (STR_EQ(tag, O_SUB_DEFER))
  {
    format_list->sub_defer = true_false(datum);
  }
  else if (STR_EQ(tag, O_CONDIT_POS))
  {
    format_list->condit_pos = NEW_STRING (datum);
  }
  else if (STR_EQ(tag, O_CONDIT_NEG))
  {
    format_list->condit_neg = NEW_STRING (datum);
  }
  else if (STR_EQ(tag, O_FRMT_CONVRT))
  {
    format_list->format_convert = NEW_STRING (datum);
  }
  else if (STR_EQ(tag, O_SUB_FORMAT))
  {
    format_list->sub_format = NEW_STRING (datum);
  }
  else if (STR_EQ(tag, O_TITLE))
  {
    format_list->subitem = make_subitem (format_list->subitem, tag, datum);
  }
  else if (STR_EQ(tag, O_DATA))
  {
    format_list->subitem = make_subitem (format_list->subitem, tag, datum);
  }
  else if (STR_EQ(tag, O_AUTH_ELEM))
  {
    format_list->subitem = make_subitem (format_list->subitem, tag, datum);
  }
  else if (STR_EQ(tag, O_MULTI_DELIM))
  {
    format_list->multi_delim = NEW_STRING (datum);
  }
}


/* ----------------------- PUBLIC FUNCTIONS ------------------- */

/* parses the output */
output_format_struct *
encode_output_format(format_list, file)
  output_format_struct  *format_list;
  char                  *file;

{
  FILE                  *fp             = NULL;
  char                  line[BUFSIZ];
  char                  tag[BUFSIZ];
  char                  datum[BUFSIZ];
  output_format_struct  *cur            = NULL;
  output_format_struct  *new            = NULL;

  if ((fp = fopen (file, "r")) == NULL)
  {
    print_error (MISC_DIAG, "encode_output_format: open error on '%s'", file);
    return (NULL);
  }

  while (fgets (line, BUFSIZ, fp) != NULL)
  {
    if (new_record (line) || format_list == NULL)
    {
      if ((new = NEW_OUTPUT_FORMAT) == (output_format_struct *)NULL)
      {
        print_error (MEMORY_ALLOCATION_PROBLEM, "");
        return(NULL);
      }
      if (format_list == NULL)
      {
        cur = format_list = new;
      }
      else
      {
        cur->nxt = new;
        cur = new;
      }
    }
    if (parse_line (line, tag, datum))
    {
      stripchar (datum, '"');
      save_output_format (cur, tag, datum);
    }
  }
  fclose (fp);
  return (format_list);
}
