/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include "boolean.h"

#include <stdio.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "defines.h"
#include "hitinfo.h"
#include "parse.h"


/**************************************************************************
  Function: This procedure returns TRUE of op is Boolean FALSE if not

  By: Mark Kosters
**************************************************************************/
int
is_operator(op)
  char *op;
{
  if (STR_EQ(op,"and"))
  {
    return (LOGICAL_AND);
  }
  else if (STR_EQ(op,"or"))
  {
    return (LOGICAL_OR);
  }
  else
  {
    return(FALSE);
  }
}

/**************************************************************************
  Function: This procedures does a union of two separate queries

  Returns: a unioned list of hits

  By: Mark Kosters
**************************************************************************/
mkdb_hitinfo_struct *
Union(list1, list2)
  mkdb_hitinfo_struct   *list1;
  mkdb_hitinfo_struct   *list2;
{
  mkdb_hitinfo_struct   *rlist = NULL;
  mkdb_hitinfo_struct   *del   = list2;

  rlist = list1;

  while (list2 != NULL)
  {
    rlist = store_hits(rlist, list2->file_no, list2->offset, FALSE);
    list2 = list2->nxt;
  }

  clear_hits (del);

  return (rlist);
}

/**************************************************************************
  Function: This procedures does an intersection of two separate queries

  Returns: a list of hits

  By: Mark Kosters
**************************************************************************/
mkdb_hitinfo_struct *
Intersection(list1, list2)
  mkdb_hitinfo_struct   *list1;
  mkdb_hitinfo_struct   *list2;
{
  mkdb_hitinfo_struct   *rlist = NULL;
  mkdb_hitinfo_struct   *del1  = list1;
  mkdb_hitinfo_struct   *del2  = list2;

  while (list2 != NULL)
  {
    while (list1 != NULL)
    {
      if (list1->file_no == list2->file_no && list1->offset == list2->offset)
      {
        rlist = store_hits (rlist, list2->file_no, list2->offset, FALSE);
        break;
      }

      list1 = list1->nxt;
    }

    list2 = list2->nxt;
  }

  clear_hits(del1);
  clear_hits(del2);

  return (rlist);
}
