/************************************************************************
RWhois Software

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and
that no charge is made for such copies.

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By:
  Mark Kosters
***************************************************************************/
#include "get_record.h"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>

#include "defines.h"
#include "fileinfo.h"
#include "log.h"
#include "misc.h"
#include "strutil.h"

#define MAX_BUF 4096

/*************************************************************************
  get_record
    based on offset, fileno, returns buf
**************************************************************************/
char *
MKget_record(offset, file_no, buf, buf_size, max_lines)
  long offset;
  int  file_no;
  char *buf;
  int *buf_size;
  int max_lines;
{
  FILE                  *fp;
  char                  line[MAX_LINE];
  mkdb_fileinfo_struct  *cur;
  int                   end_of_rec  = FALSE;
  int                   line_cnt    = 0;

  if (!*buf_size &&
      (buf = (char *) calloc ((unsigned)MAX_BUF, 
                              (unsigned)sizeof (char))) == NULL)
  {
    log_error("MKget_record: error requesting memory");
  }

  *buf_size = MAX_BUF;
  *buf = '\0';

  if ((cur = get_fileinfo (ENUM_DATA_FILE, file_no)) != NULL)
  {
    if ((fp = fopen (cur->filename, "r")) == NULL )
    {
      log_error("Unable to open %s: %s", cur->filename, l_strerror(errno));
      return (buf);
    }

    fseek (fp, offset, 0); /* SEEK_SET */
    while (!end_of_rec && fgets (line,MAX_LINE, fp) != NULL)
    {
      line_cnt++;
      /****
        restriction for max lines on referrals
        ****/
      if (max_lines > 0 && line_cnt > max_lines)
      {
        return (buf);
      }

      stripchar (line, '\n');

      if (record_separator(line))
      {
        end_of_rec = TRUE;
      }
      else
      {
        if (strlen(buf)+strlen(line) >= *buf_size)
        {
          if ((buf = (char *)realloc (buf, *buf_size*2)) == NULL)
          {
            log_error("MKget_record: error requesting memory\n");
          }

          *buf_size *= 2;
        }

        strcat(buf, line);
        strcat(buf, "\n");
      }
    }

    fclose (fp);
  }

  return (buf);
}
