/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "displays.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "attributes.h"
#include "client_info.h"
#include "client_msgs.h"
#include "defines.h"
#include "display_conf.h"
#include "dump.h"
#include "format_convert.h"
#include "forward.h"
#include "misc.h"
#include "objects.h"
#include "output.h"
#include "strutil.h"
#include "variables.h"
#include "whois.h"


/* ------------------- LOCAL FUNCTIONS --------------------- */

/* returns the header, footer and delimiters; sets pointers to NULL if
   'command' not found. */
static void
read_header_and_footers(command, header, footer, delim)
  char *command;
  char **header;
  char **footer;
  char **delim;
{
  output_command_struct *cur = return_all_defined_displays();

  while (cur != NULL)
  {
    if (cur->command != NULL && STR_EQ(cur->command, command))
    {
      *header = cur->header;
      *footer = cur->footer;
      *delim  = cur->record_delim;
      return;
    }
    cur = cur->nxt;
  }
  
  *header = NULL;
  *footer = NULL;
  *delim  = NULL;
}


/* returns a pointer to the specified format in command; NULL if does
     not exist */
static output_format_struct *
retrieve_output_format(display_list, command)
  output_display_struct *display_list;
  char                  *command;

{
  int length;

  if (!command) return NULL;
  
  length = strlen(command);

  while (display_list != NULL) {
    if ((length >= display_list->command_len) &&
        STRN_EQ(display_list->command, command, length))
    {
      return (display_list->format);
    }

    display_list = display_list->nxt;
  }

  return (NULL);
}

/* If condit_pos set, makes sure that condit_pos field in record.  If
     condit_neg set, make sure that condit_neg field not in record
     returns TRUE on success, FALSE on failure */
static int
required_fields_in_record(fields, condit_pos, condit_neg)
  field_struct  *fields;
  char          *condit_pos;
  char          *condit_neg;
{
  attribute_struct  *attribute_info;
  field_struct      *cur;
  int               done    = 0;
  int               found_pos;
  int               found_neg;

  /* neither condition set is definately true */
  if (condit_pos == NULL && condit_neg == NULL)
  {
    return (TRUE);
  }

  /* if we are making a positive assertion, search for that record */
  cur       = fields;
  found_pos = FALSE;
  found_neg = FALSE;
  
  while (cur && !done)
  {
    attribute_info = get_attribute_info_from_internal_id(cur->id);
    
    if (attribute_info)
    {
      if (condit_pos && STR_EQ(condit_pos, attribute_info->name))
      {
        found_pos = TRUE;
      }
      else if (condit_neg && STR_EQ(condit_neg, attribute_info->name))
      {
        found_neg = TRUE;
      }

      if ((found_pos && found_neg)   ||
          (!condit_pos && found_neg) ||
          (!condit_neg && found_pos))
      { 
        return TRUE;
      }
        
      cur = cur->nxt;
    }
  }

  /* reached end of list without satisfying condition */
  return FALSE;
}

/* displays a character */
static char *
display_char(c)
  char *c;
{
  if (*c == '\\' &&
      *(c + 1) != '\0' &&
      *(c + 1) == 'n') {
    /* carrriage return */
    printf("\n");
    c++;
  }
  else {
    printf("%c", *c);
  }
  return(c);
}

/* displays text string */
void
display_text(str, fixed_len)
  char  *str;
  int   fixed_len;
{
  char  *c  = str;
  int   cnt = 0;

  if (!str) return;
  
  while (*c)
  {
    c = display_char(c);
    c++;
    cnt++;
  }
  
  if (fixed_len > 0 && cnt < fixed_len)
  {
    while (cnt < fixed_len)
    {
      display_char(" ");
      cnt++;
    }
  }
}

/* displays conditional text -- or will, at least */
static void
display_conditional_text(str, fields)
  char          *str;
  field_struct  *fields;
{
  printf ("display_conditional_text: do this yet!\n");
}

/* opens file and displays contents. Returns TRUE on success, FALSE on
     failure */
static int
display_file(fname)
  char *fname;
{
  FILE *fp;
  char line[MAX_LINE];

  if ((fp = fopen (fname, "r")) == NULL)
  {
    return FALSE;
  }

  while (fgets (line, MAX_LINE, fp) != NULL)
  {
    stripchar(line, '\n');
    printf("%s\n", line);
  }

  fclose (fp);

  return TRUE;
}

/* opens file and displays contents of mime file. Returns TRUE on success,
     FALSE on failure. */
static void
display_mime(fname)
  char  *fname;
{
  printf("%%display activate:mime\n");
  display_file(fname);
  printf("%%display\n");
}

/* prints out a referral record. Expects in "host:port:server_type
   auth_str", or in URL format */
static void
display_referral(data, auth_str)
  char *data;
  char *auth_str;
{
  if (forward_status()) {
    forward_request(data, original_query(), auth_str);
  }
  else if (STRN_EQ(data, "url", 3))
  {
    /* url's do not have aa's */
    printf ("%%referral %s\n", data);
  }
  else if (auth_str != NULL)
  {
    printf ("%%referral %s %s\n", data, auth_str);
  }
  else
  {
    printf ("%%referral %s\n", data);
  }
}

/* formats and starts the subquery */
static int
run_sub_query(format, query)
  char  *format;
  char  *query;
{
  char sub_query[MAX_LINE];

  if (format != NULL)
  {
    sprintf(sub_query, "%s %s", format, query);
  }
  else
  {
    strcpy(sub_query, query);
  }

  return (whois(sub_query, FALSE, TRUE));
}

/* Tries to find internal see alsos. If dereference is on, prints only
     datum, does not try to do subquery. expects in
     host:port:server_type:<query> or host:port:<query> formats */
static void
do_see_also(data, format, defer, deref)
  char  *data;
  char  *format;
  int   defer;
  int   deref;

{
  char query[MAX_LINE];
  char location[MAX_LINE];

  separate_location_from_query(data, location, query);
  if (deref || defer)
  {
    display_text(query, 0);
  }
  else if (location_is_local (location))
  {
    run_sub_query (format, query);
  }
  else
  {
    if (format != NULL)
    {
      printf ("%%see-also %s:%s %s\n", location, format, query);
    }
    else
    {
      printf ("%%see-also %s:%s\n", location, query);
    }
  }
}

/* displays an attribute */
static void
display_attribute(attribute_info, data, auth_str, len, 
                  format_convert, deref, defer, format)
  attribute_struct  *attribute_info;
  char              *data;
  char              *auth_str;
  int               len;
  char              *format_convert;
  int               deref;
  int               defer;
  char              *format;

{
  switch (attribute_info->type)
  {
  case TYPE_TEXT:
    if (format_convert) {
      display_text_w_format_convert (format_convert, data, len);
    }
    else {
      display_text (data, len);
    }
    break;
  case TYPE_FILE:
    display_file(data);
    break;
  case TYPE_MIME:
    if (check_capabilities("mime")) {
      display_mime (data);
    }
    break;
  case TYPE_SEE_ALSO:
    if (defer && !deref) {
      return;
    }
    do_see_also(data, format, defer, deref);
    break;
  case TYPE_REFERRAL:
    display_referral(data, auth_str);
    break;
  }
}

/* sets up the the deferred subqueries. Returns a new element on the
     see-also list as a string value. */
static llist_struct *
setup_deferred_subqueries(list, data, format)
  llist_struct  *list;
  char          *data;
  char          *format;
{
  char  str[MAX_LINE];
  char  word[MAX_LINE];
  char  *new;
  char  *cp;
  char  *next;

  if (format == NULL)
  {
    strcpy (str, data);
  }
  else
  {
    /* skip over referral part */
    cp = data;
    next = get_word(cp, word);
    sprintf(str, "%s %s %s", word, format, next);
  }

  new = NEW_STRING(str);
  list = append_llist(list, new);

  return (list);
}

/* prints out the individual field */
static llist_struct *
display_field(output_format, object_name, fields, auth_str)
  output_format_struct  *output_format;
  char                  *object_name;
  field_struct          *fields;
  char                  *auth_str;

{
  field_struct      *cur                    = fields;
  attribute_struct  *attribute_info;
  char              *multi_delim;
  char              *fieldname;
  int               fixed_len;
  int               cnt;
  int               found                   = FALSE;
  int               sub_defer;
  int               sub_deref;
  char              *sub_format;
  char              *format_convert;
  llist_struct      *deferred_subqueries    = NULL;
  char              datafldname[MAX_LINE];

  /* first make sure the output format exists int the field list */
  cur       = fields;
  fieldname = output_format->value;

  while (cur != NULL && !found)
  {
    strcpy(datafldname, get_attribute_name(cur->id));
    if (STR_EQ(datafldname, fieldname))
    {
      found = TRUE;
    }

    cur = cur->nxt;
  }

  if (!found)
  {
    return (NULL);
  }

  /* check out required fields are there! if not - return */

  if (!required_fields_in_record(fields, output_format->condit_pos,
                                 output_format->condit_neg))
  {
    return NULL;
  }

  /* go for it */

  multi_delim    = output_format->multi_delim;
  fixed_len      = output_format->fixed_len;
  sub_deref      = output_format->sub_deref;
  sub_defer      = output_format->sub_defer;
  sub_format     = output_format->sub_format;
  output_format  = output_format->subitem;
  format_convert = output_format->format_convert;

  while (output_format != NULL)
  {
    switch (output_format->type)
    {
    case FORMAT_TEXT:
      display_text(output_format->value, 0);
      break;
    case FORMAT_AUTH_ELEM:
      display_text(auth_str, 0);
      break;
    case FORMAT_DATA:
      cur = fields;
      cnt = 0;
      while (cur != NULL)
      {
        attribute_info = get_attribute_info_from_internal_id(cur->id);

        if (STR_EQ(attribute_info->name, fieldname))
        { 
          if (cnt > 0)
          {
            display_text(multi_delim, 0);
          }
          display_attribute(attribute_info, cur->data, auth_str, fixed_len, 
                            format_convert, sub_deref, sub_defer, sub_format);
          if (sub_defer)
            deferred_subqueries = setup_deferred_subqueries 
                                  (deferred_subqueries, cur->data, sub_format);
          cnt++;
        }
        cur = cur->nxt;
      }
      break;
    default:
      printf("display_field: oops don't have value %d\n", 
             output_format->type);
    }

    output_format = output_format->nxt;
  }
  
  return (deferred_subqueries);
}

/* prints out the record by decoding the output format */
static void
display_record(output_format, object_name, record, delim, auth_str)
  output_format_struct  *output_format;
  char                  *object_name;
  record_struct         *record;
  char                  *delim;
  char                  *auth_str;

{
  llist_struct  *deferred_queries;
  llist_struct  *top                = NULL;
  llist_struct  *cur                = NULL;

  while (output_format != NULL)
  {
    switch (output_format->type)
    {
    case FORMAT_FIELD_DEF:
      if ((deferred_queries = display_field(output_format, object_name,
                                            record->fields, auth_str)) != NULL)
      {
        if (top == NULL)
        {
          cur = deferred_queries;
          top = deferred_queries;
        }
        else
        {
          cur->nxt = deferred_queries;
          cur      = deferred_queries;
        }
      }
      break;
    case FORMAT_TEXT:
      display_text(output_format->value, 0);
      break;
    case FORMAT_CONDIT_TEXT:
      display_conditional_text(output_format, record->fields);
      break;
    case FORMAT_AUTH_ELEM:
    case FORMAT_DATA:
    case FORMAT_SUB_FORMAT:
    case FORMAT_SUB_DEFER:
    case FORMAT_SUB_DEREF:
    default:
      break;
    }

    output_format = output_format->nxt;
  }

  cur = top;

  while (cur != NULL)
  {
    if (delim != NULL)
    {
      printf("%s\n", delim);
    }

    run_sub_query(NULL, cur->datum);

    free(cur->datum);
    cur = cur->nxt;
  }
  
  free_llist(top);
}


/* ------------------- PUBLIC FUNCTIONS --------------------- */

/* separates the location from the query */
int
separate_location_from_query(data, location, query)
  char *data;
  char *location;
  char *query;
{
  char  *word;

  if (!data || !location) return FALSE;

  word = get_word(data, location);
  if (query && word && *word) {
    strcpy(query, word);
  }

  return TRUE;
}

/* FIXME: this routine should be more sophisticated */
/* checks to see if fully qualified name and port equals localhost and
      localport */
int
location_is_local(location)
  char *location;
{
  char  datum[MAX_LINE];
  char  buf[MAX_LINE];
  char  host[MAX_LINE];
  char  port_str[MAX_LINE];
  int   port;
  char  *local_host;
  int   local_port;
  
  /* should handle URL style references here */
  
  if (parse_line(location, host, datum))
  {
    strcpy(buf, datum);
    
    if (!parse_line(buf, port_str, datum))
    {
      strcpy(port_str, buf);
    }

    port = atoi(port_str);
    local_host = get_local_hostname();
    local_port = get_port();

    if (STR_EQ(host, "localhost") && port == local_port)
    {
      return TRUE;
    }

    if (local_host && STR_EQ(host, local_host) && port == local_port)
    {
      return TRUE;
    }
  }
  return (FALSE);
}


/* retrieve results from defined formats */
int
display_results(records, cache, mode, auth_str, sub_query)
  record_struct *records;
  int           cache;
  char          *mode;
  char          *auth_str;
  int           sub_query;

{
  record_struct         *ptr            = records;
  object_struct         *object;
  output_format_struct  *output_format;
  int                   multiple        = FALSE;
  int                   cnt             = 0;
  char                  *output_type;
  char                  *header;
  char                  *footer;
  char                  *rec_separator;

  while (ptr != NULL && !multiple)
  {
    object = get_object_from_record(ptr->fields);

    if (object != NULL && object->type != REFERRAL)
    {
      cnt++;

      if (cnt > 1)
      {
        multiple = TRUE;
      }
    }

    ptr = ptr->nxt;
  }

  /* Define the type of display */
  if (mode) {
    output_type = mode;
  }
  else if (multiple) {
    output_type = get_display_status(MULTI_DISPLAY);
  }
  else {
    output_type = get_display_status(SINGLE_DISPLAY);
  }

  read_header_and_footers(output_type, &header, &footer, &rec_separator);
  
  if (header != NULL && cnt && !sub_query) {
    printf ("%s\n", header);
  }

  cnt = 0;

  while (records != NULL)
  {
    cnt++;
    object = get_object_from_record(records->fields);

    if (object != NULL)
    {
      if (rec_separator!=NULL && (!sub_query || cnt > 1)) {
        printf ("%s\n", rec_separator);
      }
      if (STR_EQ(output_type, "dump")) {
        display_dump_format (object->name, records, cnt-1);
      }
      else if ((output_format = retrieve_output_format(object->output_format,
                                                       output_type)) == NULL)
      {
        print_error(MISC_DIAG, "No format for Object '%s'", object->name);
        display_dump_format(object->name, records, cnt - 1);
      }
      else {
        display_record(output_format, object->name, records, rec_separator, 
                       auth_str);
      }
    }
    else {
      print_error(UNIDENT_ERROR, "can not find object!");
    }
    if (cache) {
      print_error(NON_AUTH_ANSWER,"");
    }
   
    records = records->nxt;
  }

  if (footer != NULL && cnt && !sub_query){
    printf ("%s\n", footer);
  }

  return (cnt);
}
