/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "initialize.h"

#include <stdio.h>
#include <signal.h>

#include "client_msgs.h"
#include "commands.h"
#include "defines.h"
#include "directive.h"
#include "read_config.h"
#include "session.h"

/* these are needed for the static commands */
#include "help.h"
#include "exit.h"

/* local prototypes */

static void initialize_signals PROTO(());

static void initialize_command_list PROTO(());

RETSIGTYPE int_exit PROTO(());

/* initialize_environment: sets up static configuration parameters
     (directive list, etc) and sets up all signal handlers */
int
initialize_environment()
{
  initialize_signals();
  initialize_command_list();
  initialize_directive_list();

  return TRUE;
}

/* the exit signal handler */
RETSIGTYPE
int_exit()
{
  exit_program();
}

/* Set a signal handler up so that we always exit gracefully */
static void
initialize_signals()
{
  signal(SIGKILL, int_exit);
  signal(SIGTERM, int_exit);
  signal(SIGPIPE, int_exit);
  signal(SIGINT,  int_exit);
}


/****************************************************************************
 initializes the internal commands
****************************************************************************/
static void
initialize_command_list()
{
  append_command (NULL, 0, "!", "handle query", TRUE, COMMAND_QUERY, NULL, 
                  NULL, NULL);
  append_command ("help", 4, NULL, "display help text", FALSE, 
                  COMMAND_SPECIAL, NULL, help, NULL);
  append_command ("quit", 4, NULL, "exit rwhois", FALSE, COMMAND_SPECIAL, NULL,
                  exit_whois, NULL);
  append_command ("exit", 4, NULL, "exit rwhois", FALSE, COMMAND_SPECIAL, NULL,
                  exit_whois, NULL);
}

