/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "security.h"

#include <pwd.h>
#include <sys/types.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif /* HAVE_UNISTD_H */

#include "log.h"
#include "read_config.h"
#include "variables.h"

#if AUTHORIZATION_ON
#include <stdio.h>
#include "../tools/tcpd_wrapper/tcpd.h"
#include "defines.h"
#endif

/***************************************************************************
sets up a security playground by doing a chroot and setuid (taken
from W.Z. Venema's chrootuid code.

returns TRUE if ok
        FALSE otherwise
****************************************************************************/
int
setup_security()
{
  struct passwd *pwd;
  char          *userid;
  int           id = getuid();


  /* attempt to chroot(), if asked to do so */
  if (is_chrooted())
  {
    if (id != 0)
    {
      log_error("Must be root to perform chroot");
    }

    /* now attempt to chroot() */
    if (chroot(get_root_dir())) {
      log_error("chroot to (%s) failed: %s", get_root_dir(),
                l_strerror(errno));
      return FALSE;
    }

    /* now that we've chrooted, the root directory is now '/' only */
    set_root_dir("/");
    chdir_root_dir();
  }
  
  /* change the user id of the process, if we can */
  userid = get_process_userid();
  if (id == 0)
  {
    /* don't change user id if the value is NULL */
    if (userid && *userid)
    {
      if ((pwd = getpwnam(userid)) == 0)
      {
        log_error("'%s' user id is unknown -- unable to change id", 
                  userid);
        return FALSE;
      }

      /* change group id first */
      if (setgid(pwd->pw_gid))
      {
        log_error("setup_security: setgid failed");
        return FALSE;
      }
      /* now change user id */
      if (setuid(pwd->pw_uid))
      {
        log_error("setup_security: setuid failed");
        return (FALSE);
      }
      /* In case we still have the /etc/passwd file still open. */
      endpwent();
    }
    else
    {
      log_error("Warning: running as root!");
    }
  }

  /* if we got this far, everything went OK */
  return TRUE;
}

/****************************************************************************
 restricts who can run the command
   returns TRUE if can
           FALSE if not
****************************************************************************/
int
authorized_directive (directive)
  char *directive;
{
  char              *hosts_allow;
  char              *hosts_deny;
  
  hosts_allow = get_security_allow();
  hosts_deny  = get_security_deny();

#if AUTHORIZATION_ON
  {
    /* hosts_access from tcp_wrapper changed its interface to make it
       harder to change the access list files.  Fortunately, they are
       external global variables */
    extern char *hosts_allow_table;
    extern char *hosts_deny_table;
    struct request_info     req;

    hosts_allow_table = hosts_allow;
    hosts_deny_table  = hosts_deny;

    /* set up the request structure */
    request_init(&req, RQ_FILE, 0, RQ_DAEMON, directive);
       
    /* fill in the client info */
    fromhost(&req);

    /* return the results of the access check */
    return(hosts_access(&req));
  }
#else
  return TRUE;
#endif
  
}
