/************************************************************************
RWhois Software 

Copyright (c) 1994 Scott Williamson and Mark Kosters

Users may distribute the binary and source code to third parties provided
that the copyright notice and this statement appears on all copies and 
that no charge is made for such copies.  

This software is provided AS IS with no warranties of any kind.  The authors
shall have no liability with respect to the infringement of copyrights,
trade secrets or any patents by this file or any part thereof.  In no
event will the authors be liable for any lost revenue or profits or
other special, indirect and consequential damages.

Comments and additions should be sent to the authors:

                    Scott Williamson
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4820
                    scottw@internic.net

                    Mark Kosters
                    Network Solutions
                    505 Huntmar Park Drive
                    Herndon, VA 22071
                    (703) 742-4795
                    markk@internic.net


Developed By: 
  Mark Kosters
***************************************************************************/
#include "soa.h"

#include <stdio.h>
#include <string.h>
#ifdef HAVE_STRINGS_H
#include <strings.h>
#endif /* HAVE_STRINGS_H */

#include "attributes.h"
#include "client_msgs.h"
#include "defines.h"
#include "fileinfo.h"
#include "holdconnect.h"
#include "misc.h"
#include "objects.h"
#include "parse.h"
#include "strutil.h"
#include "whois.h"
#include "xfer.h"

/*******************
   local protocols 
********************/
static int open_mkdb_soa PROTO((char *object_name, char *filename,
                                   char *aa, char *soa));

static int show_all_soa();

static int show_soa_contents PROTO((char *object_name, char *filename,
                                     char *aa, char *soa));

static void print_out_soa PROTO((char *object_name, char *buf));

static char *ToLower PROTO((char *str));

/****************************************************************************
  displays the soa of a particular authority item
****************************************************************************/
int
show_soa (str)
  char *str;
{
  query_item_struct *query_list;
  int               hits;
  int               null_query = FALSE;
  
  if (!str || !*str)
  {
    null_query = TRUE;
  }

  if (null_query)
  {
    hits = show_all_soa();
    if (!hits)
    {
      print_error(NO_RECORDS_FOUND, "");
      return FALSE;
    }
  }
  else
  {
    query_list = encode_command(str);
  
    hits = run_query(SOA, FALSE, 0, query_list, FALSE);
  }

  if (hits)
  {
    print_ok();
  }
  else
  {
    print_error(NO_SOA, "");
    return FALSE;
  }
  record_results(FALSE, FALSE, hits);

  return TRUE;
}


/* show_all_soa(): display all the soa records */


static int
show_all_soa()
{
  object_struct *obj;
  char      filename[MAX_LINE];
  int       hits    = 0;

  obj = get_object_by_object_name("soa");

  sprintf (filename, "%s.db", obj->local_db);
  hits = open_mkdb_soa(obj->name, filename, NULL, NULL);

#ifdef DEBUG
  printf("number of soa:%d\n", hits);
#endif

  return hits; 
}



/****************************************************************************
  opens the source files -- return the total number of records
****************************************************************************/
static int
open_mkdb_soa (object_name, filename, aa, soa)
  char  *object_name;
  char  *filename;
  char  *aa;
  char  *soa;   

{
  char  *fname;
  int   reset;
  int   hits    =0;
  int   tothits =0;

  if (!read_fileinfo (filename)) 
  {
    /* no data file just means that there are no SOA records */
    return 0;
  }

  reset = 1;
  while ((fname = get_fileinfo_data_files (reset)) != NULL) 
  {
    reset = 0;
    show_soa_contents (object_name, fname, NULL, NULL);
    hits = get_fileinfo_record_count ();
    if (hits) 
    {
      printf ("%%soa\n");  
    }
    tothits += hits;
  }
  
  clear_fileinfo();

  return (hits);
}

/****************************************************************************
  displays the records of soa
****************************************************************************/
static int
show_soa_contents (object_name, filename, aa, soa)
  char  *object_name;
  char  *filename;
  char  *aa;
  char  *soa;

{
  FILE  *fp;
  char  buf[4096];
  char  line[MAX_LINE];
  int   ret =FALSE;
  int   addlines=TRUE;

  if ((fp = fopen (filename, "r")) == NULL) 
  {
/*     print_error (MISC_DIAG, "Can not open data file '%s'", filename); */
    return;
  }
  *buf= '\0';

  while (fgets (line, MAX_LINE, fp) != NULL) 
  {
    stripchar (line, '\n');
    if (record_separator (line)) 
    {
      if (strlen(buf)) 
      {
        if (ret) 
        {
          printf ("%%soa\n");
        }       
        print_out_soa (object_name, buf);
      
        if (!addlines) 
        {
          print_error (MISC_DIAG, "record truncated");
    }
      
        ret = TRUE;
      }

      *buf = '\0';
    }
    else if (addlines) 
    {
      if (strlen (buf) + strlen (line) + 2 > 4096)
         addlines = FALSE;
      else 
      {
        strcat (buf, "\n");
        strcat (buf, line);
      }
    }
  }

  if (strlen(buf)) 
  {
    if (ret)
      printf ("%%soa\n");
  
    print_out_soa (object_name, buf);

    ret = TRUE;
    *buf = '\0';
  }

  fclose (fp);
  return (ret);
}


/****************************************************************************
  prints out the soa stuff
****************************************************************************/

static void print_out_soa (object_name, buf)
  char  *object_name;
  char  *buf;

{
  int   cnt =0;
  char  line[MAX_LINE];
  char  object[MAX_LINE];
  char  attribute[MAX_LINE];
  char  datum[MAX_LINE];

  set_attributes_list_from_object (FALSE, NULL, NULL);

  while (delimstr(buf, "\n", line, cnt++)) 
  {
    if (get_tuple(object, attribute, datum, line))
      printf ("%%%s %s: %s\n", object, ToLower(attribute), datum);
  }
}


/***************
   convert str to lower case
 ***************/
static char *
ToLower(str)
  char  *str;
{
  int i;
  if(!str || !*str)
    return ((char *) NULL);
  
  for(i=0; str[i]; i++)
    if(str[i]>='A' && str[i]<='Z')
      str[i]=str[i]-'A'+'a'; 
  return str;
}

